@@ -22,6 +22,7 @@
 
 #include "kglobalshortcuttest.h"
 #include <qdbusinterface.h>
+#include <QSignalSpy>
 #include <QTest>
 #include <QAction>
 #include <QThread>
@@ -30,25 +31,39 @@
 
 #include <qplatformdefs.h>
 
-#include <QtDBus/QDBusConnectionInterface>
+#include <QDBusConnectionInterface>
 
-const QKeySequence sequenceA = QKeySequence(Qt::SHIFT + Qt::META + Qt::CTRL + Qt::ALT + Qt::Key_F28);
+#ifdef HAVE_XCB_XTEST
+#include <QX11Info>
+#define XK_MISCELLANY
+#define XK_XKB_KEYS
+#include <X11/keysymdef.h>
+#include <xcb/xcb_keysyms.h>
+#include <xcb/xtest.h>
+#endif
+
+/*
+ * NOTE
+ * ----
+ * These tests could be better. They don't include actually triggering actions,
+ * and we just choose very improbable shortcuts to avoid conflicts with real
+ * applications' shortcuts.
+ *
+*/
+
+const QKeySequence sequenceA = QKeySequence(Qt::SHIFT + Qt::META + Qt::CTRL + Qt::ALT + Qt::Key_F12);
 const QKeySequence sequenceB = QKeySequence(Qt::Key_F29);
 const QKeySequence sequenceC = QKeySequence(Qt::SHIFT + Qt::META + Qt::CTRL + Qt::Key_F28);
 const QKeySequence sequenceD = QKeySequence(Qt::META + Qt::ALT + Qt::Key_F30);
 const QKeySequence sequenceE = QKeySequence(Qt::META + Qt::Key_F29);
 const QKeySequence sequenceF = QKeySequence(Qt::META + Qt::Key_F27);
 
-/* These tests could be better. They don't include actually triggering actions,
-   and we just choose very improbable shortcuts to avoid conflicts with real
-   applications' shortcuts. */
-
 //we need a GUI so that the implementation can grab keys
 QTEST_MAIN(KGlobalShortcutTest)
 
 void KGlobalShortcutTest::initTestCase()
 {
-    QStandardPaths::enableTestMode(true);
+    QStandardPaths::setTestModeEnabled(true);
     m_daemonInstalled = true;
 
     QDBusConnectionInterface *bus = QDBusConnection::sessionBus().interface();
@@ -60,8 +75,24 @@ void KGlobalShortcutTest::initTestCase()
     }
 }
 
-void KGlobalShortcutTest::setupTest(QString id)
+/**
+ * NOTE: This method alters a KDE config file in your home directory.
+ *
+ * KDE4: ~/.kde4/share/config/kglobalshortcutsrc
+ * KF5: ~/.config/kglobalshortcutsrc
+ *
+ * At least on KDE4 this file cannot be modified by hand if the Plasma session
+ * is running. So in case you have to clean the file of spurious entries
+ * (e.g. because of broken or failing test cases)
+ * you have to logout from Plasma.
+ *
+ * The following sections are created and normally removed automatically:
+ * [qttest]
+ */
+void KGlobalShortcutTest::setupTest(const QString& id)
 {
+    QString componentName = "qttest";
+
     if (m_actionA) {
         KGlobalAccel::self()->removeAllShortcuts(m_actionA);
         delete m_actionA;
@@ -73,24 +104,24 @@ void KGlobalShortcutTest::setupTest(QString id)
     }
 
     // Ensure that the previous test did cleanup correctly
+#if KGLOBALACCEL_ENABLE_DEPRECATED_SINCE(4, 2)
     KGlobalAccel *kga = KGlobalAccel::self();
-#ifndef KGLOBALACCEL_NO_DEPRECATED
     QList<QStringList> components = kga->allMainComponents();
     QStringList componentId;
-    componentId << "qttest" << QString() << "KDE Test Program" << QString();
+    componentId << componentName << QString() << "KDE Test Program" << QString();
     // QVERIFY(!components.contains(componentId));
 #endif
 
     m_actionA = new QAction("Text For Action A", this);
     m_actionA->setObjectName("Action A:" + id);
-    m_actionA->setProperty("componentName", "qttest");
+    m_actionA->setProperty("componentName", componentName);
     m_actionA->setProperty("componentDisplayName", "KDE Test Program");
     KGlobalAccel::self()->setShortcut(m_actionA, QList<QKeySequence>() << sequenceA << sequenceB, KGlobalAccel::NoAutoloading);
     KGlobalAccel::self()->setDefaultShortcut(m_actionA, QList<QKeySequence>() << sequenceA << sequenceB, KGlobalAccel::NoAutoloading);
 
     m_actionB = new QAction("Text For Action B", this);
     m_actionB->setObjectName("Action B:" + id);
-    m_actionB->setProperty("componentName", "qttest");
+    m_actionB->setProperty("componentName", componentName);
     m_actionA->setProperty("componentDisplayName", "KDE Test Program");
     KGlobalAccel::self()->setShortcut(m_actionB, QList<QKeySequence>(), KGlobalAccel::NoAutoloading);
     KGlobalAccel::self()->setDefaultShortcut(m_actionB, QList<QKeySequence>(), KGlobalAccel::NoAutoloading);
@@ -107,6 +138,7 @@ void KGlobalShortcutTest::testSetShortcut()
     // Just ensure that the desired values are set for both actions
     QList<QKeySequence> cutA;
     cutA << sequenceA << sequenceB;
+
     QCOMPARE(KGlobalAccel::self()->shortcut(m_actionA), cutA);
     QCOMPARE(KGlobalAccel::self()->defaultShortcut(m_actionA), cutA);
 
@@ -114,6 +146,57 @@ void KGlobalShortcutTest::testSetShortcut()
     QVERIFY(KGlobalAccel::self()->defaultShortcut(m_actionB).isEmpty());
 }
 
+void KGlobalShortcutTest::testActivateShortcut()
+{
+#ifdef HAVE_XCB_XTEST
+    if (!QX11Info::isPlatformX11()) {
+        QSKIP("This test can only be run on platform xcb");
+    }
+
+    setupTest("testActivateShortcut");
+    if (!m_daemonInstalled) {
+        QSKIP("kglobalaccel not installed");
+    }
+    QSignalSpy actionASpy(m_actionA, SIGNAL(triggered(bool)));
+    QVERIFY(actionASpy.isValid());
+
+    xcb_connection_t *c = QX11Info::connection();
+    xcb_window_t w = QX11Info::appRootWindow();
+
+    xcb_key_symbols_t *syms = xcb_key_symbols_alloc(c);
+    auto getCode = [syms] (int code) {
+        xcb_keycode_t *keyCodes = xcb_key_symbols_get_keycode(syms, code);
+        const xcb_keycode_t ret = keyCodes[0];
+        free(keyCodes);
+        return ret;
+    };
+    const xcb_keycode_t shift = getCode(XK_Shift_L);
+    const xcb_keycode_t meta = getCode(XK_Super_L);
+    const xcb_keycode_t control = getCode(XK_Control_L);
+    const xcb_keycode_t alt = getCode(XK_Alt_L);
+    const xcb_keycode_t f12 = getCode(XK_F12);
+    xcb_key_symbols_free(syms);
+
+    xcb_test_fake_input(c, XCB_KEY_PRESS, meta,    XCB_TIME_CURRENT_TIME, w, 0, 0, 0);
+    xcb_test_fake_input(c, XCB_KEY_PRESS, control, XCB_TIME_CURRENT_TIME, w, 0, 0, 0);
+    xcb_test_fake_input(c, XCB_KEY_PRESS, alt,     XCB_TIME_CURRENT_TIME, w, 0, 0, 0);
+    xcb_test_fake_input(c, XCB_KEY_PRESS, shift,   XCB_TIME_CURRENT_TIME, w, 0, 0, 0);
+    xcb_test_fake_input(c, XCB_KEY_PRESS, f12,     XCB_TIME_CURRENT_TIME, w, 0, 0, 0);
+
+    xcb_test_fake_input(c, XCB_KEY_RELEASE, f12,     XCB_TIME_CURRENT_TIME, w, 0, 0, 0);
+    xcb_test_fake_input(c, XCB_KEY_RELEASE, shift,   XCB_TIME_CURRENT_TIME, w, 0, 0, 0);
+    xcb_test_fake_input(c, XCB_KEY_RELEASE, meta,    XCB_TIME_CURRENT_TIME, w, 0, 0, 0);
+    xcb_test_fake_input(c, XCB_KEY_RELEASE, control, XCB_TIME_CURRENT_TIME, w, 0, 0, 0);
+    xcb_test_fake_input(c, XCB_KEY_RELEASE, alt,     XCB_TIME_CURRENT_TIME, w, 0, 0, 0);
+    xcb_flush(c);
+
+    QVERIFY(actionASpy.wait());
+    QCOMPARE(actionASpy.count(), 1);
+#else
+    QSKIP("This test requires to be compiled with XCB-XTEST");
+#endif
+}
+
 // Current state
 // m_actionA: (sequenceA, sequenceB)
 // m_actionB: (,)
@@ -260,8 +343,8 @@ void KGlobalShortcutTest::testListActions()
     }
 
     // As in kdebase/workspace/kcontrol/keys/globalshortcuts.cpp
+#if KGLOBALACCEL_ENABLE_DEPRECATED_SINCE(4, 2)
     KGlobalAccel *kga = KGlobalAccel::self();
-#ifndef KGLOBALACCEL_NO_DEPRECATED
     QList<QStringList> components = kga->allMainComponents();
     //qDebug() << components;
     QStringList componentId;
@@ -269,11 +352,13 @@ void KGlobalShortcutTest::testListActions()
     QVERIFY(components.contains(componentId));
 #endif
 
-#ifndef KGLOBALACCEL_NO_DEPRECATED
+#if KGLOBALACCEL_ENABLE_DEPRECATED_SINCE(4, 2)
     QList<QStringList> actions = kga->allActionsForComponent(componentId);
     QVERIFY(!actions.isEmpty());
-    QStringList actionIdA; actionIdA << "qttest" << "Action A:testListActions" << "KDE Test Program" << "Text For Action A";
-    QStringList actionIdB; actionIdB << "qttest" << "Action B:testListActions" << "KDE Test Program" << "Text For Action B";
+    QStringList actionIdA;
+    actionIdA << "qttest" << "Action A:testListActions" << "KDE Test Program" << "Text For Action A";
+    QStringList actionIdB;
+    actionIdB << "qttest" << "Action B:testListActions" << "KDE Test Program" << "Text For Action B";
     //qDebug() << actions;
     QVERIFY(actions.contains(actionIdA));
     QVERIFY(actions.contains(actionIdB));
@@ -292,7 +377,7 @@ void KGlobalShortcutTest::testComponentAssignment()
      ************************************************************/
     // Action without component name get the global component
     {
-        QAction action("Text For Action A", NULL);
+        QAction action("Text For Action A", nullptr);
         action.setObjectName("Action C");
 
         QCOMPARE(action.property("componentName").toString(), QString());
@@ -304,7 +389,7 @@ void KGlobalShortcutTest::testComponentAssignment()
 
     // Action with component name keeps its component name
     {
-        QAction action("Text for Action C", 0);
+        QAction action("Text for Action C", nullptr);
         action.setObjectName("Action C");
         action.setProperty("componentName", otherComponent);
 
@@ -324,7 +409,7 @@ void KGlobalShortcutTest::testConfigurationActions()
     }
 
     // Create a configuration action
-    QAction cfg_action("Text For Action A", NULL);
+    QAction cfg_action("Text For Action A", nullptr);
     cfg_action.setObjectName("Action A:testConfigurationActions");
     cfg_action.setProperty("isConfigurationAction", true);
     cfg_action.setProperty("componentName", "qttest");
@@ -389,12 +474,28 @@ void KGlobalShortcutTest::testNotification()
     KGlobalAccel::self()->removeAllShortcuts(action);
 }
 
+void KGlobalShortcutTest::testGetGlobalShortcut()
+{
+    setupTest("testLoadShortcutFromGlobalSettings"); // cleanup see testForgetGlobalShortcut
+    if (!m_daemonInstalled) {
+        QSKIP("kglobalaccel not installed");
+    }
+
+    // retrieve shortcut list
+    auto shortcutList = KGlobalAccel::self()->globalShortcut("qttest", "Action A:testLoadShortcutFromGlobalSettings");
+    QCOMPARE(shortcutList.count(), 2); // see setupTest
+
+    // test for a real shortcut:
+//     shortcutList = KGlobalAccel::self()->shortcut("kwin", "Kill Window");
+//     QCOMPARE(shortcutList.count(), 1);
+}
+
 void KGlobalShortcutTest::testForgetGlobalShortcut()
 {
     setupTest("testForgetGlobalShortcut");
 
     // Ensure that forgetGlobalShortcut can be called on any action.
-    QAction a("Test", NULL);
+    QAction a("Test", nullptr);
     KGlobalAccel::self()->removeAllShortcuts(&a);
     if (!m_daemonInstalled) {
         QSKIP("kglobalaccel not installed");
@@ -409,8 +510,8 @@ void KGlobalShortcutTest::testForgetGlobalShortcut()
     // kglobalaccel writes asynchronous.
     QThread::sleep(1);
 
+#if KGLOBALACCEL_ENABLE_DEPRECATED_SINCE(4, 2)
     KGlobalAccel *kga = KGlobalAccel::self();
-#ifndef KGLOBALACCEL_NO_DEPRECATED
     QList<QStringList> components = kga->allMainComponents();
     QStringList componentId;
     componentId << "qttest" << QString() << "KDE Test Program" << QString();
@@ -418,3 +519,4 @@ void KGlobalShortcutTest::testForgetGlobalShortcut()
 #endif
 }
 
+
