@@ -21,9 +21,7 @@
 #include "globalshortcutcontext.h"
 #include "logging_p.h"
 
-#include <QDebug>
 #include <QProcess>
-#include <QDir>
 #include <QDBusConnectionInterface>
 
 
@@ -50,7 +48,7 @@ void KServiceActionComponent::emitGlobalShortcutPressed( const GlobalShortcut &s
     const bool klauncherAvailable = dbusDaemon->isServiceRegistered(QStringLiteral("org.kde.klauncher5"));
 
     //we can't use KRun there as it depends from KIO and would create a circular dep
-    if (shortcut.uniqueName() == QStringLiteral("_launch")) {
+    if (shortcut.uniqueName() == QLatin1String("_launch")) {
         QStringList parts = m_desktopFile.desktopGroup().readEntry(QStringLiteral("Exec"), QString()).split(QChar(' '));
         if (parts.isEmpty()) {
             return;
@@ -71,11 +69,12 @@ void KServiceActionComponent::emitGlobalShortcutPressed( const GlobalShortcut &s
 
             QDBusConnection::sessionBus().asyncCall(msg);
         } else {
-            QProcess::startDetached(m_desktopFile.desktopGroup().readEntry(QStringLiteral("Exec"), QString()));
+            QProcess::startDetached(command, parts);
         }
         return;
     }
-    foreach (const QString &action, m_desktopFile.readActions()) {
+    const auto lstActions = m_desktopFile.readActions();
+    for (const QString &action : lstActions) {
         if (action == shortcut.uniqueName()) {
             QStringList parts = m_desktopFile.actionGroup(action).readEntry(QStringLiteral("Exec"), QString()).split(QChar(' '));
 
@@ -119,8 +118,8 @@ void KServiceActionComponent::loadFromService()
 
     GlobalShortcut *shortcut = registerShortcut(QStringLiteral("_launch"), m_desktopFile.readName(), shortcutString, shortcutString);
     shortcut->setIsPresent(true);
-
-    foreach(const QString &action, m_desktopFile.readActions())
+    const auto lstActions = m_desktopFile.readActions();
+    for (const QString &action : lstActions)
         {
         shortcuts = m_desktopFile.actionGroup(action).readEntry(QStringLiteral("X-KDE-Shortcuts"), QString()).split(QChar(','));
         if (!shortcuts.isEmpty())
